// generated by Fast Light User Interface Designer (fluid) version 1.0303

#ifndef record_browse_h
#define record_browse_h
#include <FL/Fl.H>
#include "record_loader_gui.h"
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Group.H>
extern Fl_Group *tabDataFiles;
#include <FL/Fl_Input_Choice.H>
extern Fl_Input_Choice *inpDataSources;
#include <FL/Fl_Light_Button.H>
extern Fl_Light_Button *btnDataSourceUpdate;
#include <FL/Fl_Button.H>
extern Fl_Button *btnDataSourceReset;
Fl_Double_Window* make_record_loader_window();
#endif
