/*
 * This file is included by fldigirc.rc and flarqrc.rc, which must both define
 * RC_FILE_VERSION_QUAD and RC_FILE_DESCRIPTION and include config.h.
 */

#include <winver.h>

VS_VERSION_INFO VERSIONINFO
    FILEVERSION    RC_FILE_VERSION_QUAD
    PRODUCTVERSION RC_FILE_VERSION_QUAD
    FILEFLAGSMASK  VS_FFI_FILEFLAGSMASK
    FILEFLAGS      0
    FILEOS         VOS_NT_WINDOWS32
    FILETYPE       VFT_APP
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "FileDescription", RC_FILE_DESCRIPTION "\0"
            VALUE "ProductVersion", PACKAGE_VERSION "\0"
            VALUE "FileVersion", PACKAGE_VERSION "." RC_BUILD_TIME "\0"
            VALUE "InternalName", PACKAGE_TARNAME "\0"
            VALUE "OriginalFilename", PACKAGE_TARNAME ".exe\0"
            VALUE "CompanyName", "\0"
            VALUE "LegalCopyright", PACKAGE_AUTHORS "\0"
            VALUE "Licence", "GPLv3+\0"
            VALUE "Info", "http://www.w1hkj.com/\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END
