// ----------------------------------------------------------------------------
//
//	htmlstrings.cxx
//
// Copyright (C) 2008
//		Dave Freese, W1HKJ
//
// This file is part of fldigi.
//
// Fldigi is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Fldigi is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#include <config.h>

#include "dialogs/guide.cxx"

const char* szAbout =
"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n\
<html>\n\
<head>\n\
  <title>About</title>\n\
</head>\n\
<BODY BGCOLOR=FFFFCO TEXT=101010>\n\
<font size=\"0\" face=\"Verdana, Arial, Helvetica\">\n\
<CENTER>\n\
<H3><I>Fldigi " PACKAGE_VERSION "</I></H3>\n\
<br>\n\
Fast and Light DIGItal modem program\n\
</CENTER>\n\
<P>\n\
Copyright \251 2008-2014\n\
<P>\n\
Distributed under the GNU General Public License version 3 or later.<br>\n\
This is free software: you are free to change and redistribute it.<br>\n\
There is NO WARRANTY, to the extent permitted by law.\n\
<H4>Programmers:</H4>\n\
<pre>\
Dave Freese        W1HKJ\n\
Stelios Bounanos   M0GLD\n\
Rem\355 Chateauneu    F4ECW\n\
John Douyere       VK2ETA\n\
Stefan Fendt       DL1SMF\n\
Leigh Klotz        WA5ZNU\n\
John Phelps        KL4YFD\n\
Andrej Lajovic     S57LN\n\
Robert Stiles      KK5VD\n\
</pre>\
<H4>Beginners' Guide:</H4>\n\
<pre>\
Murray Greenman      ZL1BPU<br>\n\
</pre>\
<H4>Localization:</H4>\n\
<pre>\
Espa\361ol     Spanish      Pavel Milanes Costa CO7WT\n\
                         Christian W. Correa HK4QWC\n\
Deutsch     German       Marc Richter        DF2MR\n\
Fran\347ais    French       Bernard Seront      F4GAR\n\
Italiano    Italian      Pierfrancesco Caci  IK5PVX\n\
Język       Polish       Roman Bagiński      SP4JEU\n\
Nederlands  Dutch        Peter van der Post  PA1POS\n\
</pre>\
</body>\n\
</html>\n\
";

