// generated by Fast Light User Interface Designer (fluid) version 1.0302

#ifndef ghpsdr_h
#define ghpsdr_h
#include <FL/Fl.H>
#include "flslider2.h"
#include <FL/Fl_Double_Window.H>
extern Fl_Value_Input2 *SdrAudioGain;
#include <FL/Fl_Button.H>
extern Fl_Button *SaveAG;
extern Fl_Value_Input2 *SdrRFGain;
extern Fl_Button *SaveRFG;
extern Fl_Button *btnSaveConfigFile;
#include <FL/Fl_Round_Button.H>
extern Fl_Round_Button *Dither;
extern Fl_Round_Button *Random;
extern Fl_Round_Button *PreAmp;
Fl_Double_Window* ghpsdrDialogRx();
#include <FL/Fl_Roller.H>
Fl_Double_Window* ghpsdrDialogTx();
Fl_Double_Window* ghpsdrDialogAlex();
Fl_Double_Window* ghpsdrDialogMisc();
#endif
