#define BUILD_FLRADIO 1
#include "config.h"

#define IDI_ICON 101
IDI_ICON ICON DISCARDABLE "flradio.ico"

#define RC_FILE_VERSION_QUAD FLRADIO_VERSION_MAJOR,FLRADIO_VERSION_MINOR,0,0
#define RC_FILE_DESCRIPTION  "Fast Light Digital Modem Application"

#include "common.rc"
