// ----------------------------------------------------------------------------
// Copyright (C) 2014
//              David Freese, W1HKJ
//
// This file is part of fldigi
//
// fldigi is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// fldigi is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#include <config.h>

#include "fontdef.h"

#include "Feld7x7-14.cxx"
#include "Feld7x7n-14.cxx"
#include "FeldDx-14.cxx"
#include "FeldFat-14.cxx"
#include "FeldHell-12.cxx"
#include "FeldLittle-12.cxx"
#include "FeldLo8-14.cxx"
#include "FeldLow-14.cxx"
#include "FeldModern-14.cxx"
#include "FeldModern8-14.cxx"
#include "FeldNarr-14.cxx"
#include "FeldReal-14.cxx"
#include "FeldStyl-14.cxx"
#include "FeldVert-14.cxx"
#include "FeldWide-14.cxx"

char szFeldFonts[] = "\
7x7     14|\
7x7n    14|\
Dx      14|\
fat     14|\
hell    12|\
little  12|\
lo8     14|\
low     14|\
modern  14|\
modern8 14|\
narr    14|\
real    14|\
style   14|\
vert    14|\
wide    14";

